==> v0.1.tar.gz: create/src/main/java/com/simibubi/create/modules/curiosities/placementHandgun/BuilderGunHandler.java <==
package com.simibubi.create.modules.curiosities.placementHandgun;

import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.function.Supplier;

import org.lwjgl.opengl.GL11;

import com.mojang.blaze3d.platform.GlStateManager;
import com.simibubi.create.AllItems;
import com.simibubi.create.foundation.utility.TessellatorHelper;

import net.minecraft.client.Minecraft;
import net.minecraft.client.entity.player.AbstractClientPlayerEntity;
import net.minecraft.client.entity.player.ClientPlayerEntity;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.FirstPersonRenderer;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.entity.PlayerRenderer;
import net.minecraft.client.renderer.model.ItemCameraTransforms;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.client.world.ClientWorld;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.particles.ParticleTypes;
import net.minecraft.util.Hand;
import net.minecraft.util.HandSide;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.SoundEvents;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.util.math.Vec3d;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.client.event.RenderSpecificHandEvent;
import net.minecraftforge.client.event.RenderWorldLastEvent;
import net.minecraftforge.event.TickEvent.ClientTickEvent;
import net.minecraftforge.event.TickEvent.Phase;
import net.minecraftforge.event.world.BlockEvent.BreakEvent;
import net.minecraftforge.eventbus.api.EventPriority;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.common.Mod.EventBusSubscriber;

@SuppressWarnings("deprecation")
@EventBusSubscriber(value = Dist.CLIENT)
public class BuilderGunHandler {

	private static List<LaserBeam> cachedBeams;
	private static float leftHandAnimation;
	private static float rightHandAnimation;
	private static float lastLeftHandAnimation;
	private static float lastRightHandAnimation;

	private static boolean dontReequipLeft;
	private static boolean dontReequipRight;

	public static class LaserBeam {
		float itensity;
		Vec3d start;
		Vec3d end;
		boolean follow;
		boolean mainHand;

		public LaserBeam(Vec3d start, Vec3d end) {
			this.start = start;
			this.end = end;
			itensity = 1;
		}

		public LaserBeam followPlayer(boolean follow, boolean mainHand) {
			this.follow = follow;
			this.mainHand = mainHand;
			return this;
		}

		public Vec3d getStart() {
			if (follow)
				return getExactBarrelPos(mainHand);
			return start;
		}
	}

	public static Vec3d getExactBarrelPos(boolean mainHand) {
		float partialTicks = Minecraft.getInstance().getRenderPartialTicks();
		ClientPlayerEntity player = Minecraft.getInstance().player;
		float yaw = (float) ((player.getYaw(partialTicks)) / -180 * Math.PI);
		float pitch = (float) ((player.getPitch(partialTicks)) / -180 * Math.PI);
		Vec3d barrelPosNoTransform = new Vec3d(mainHand == (player.getPrimaryHand() == HandSide.RIGHT) ? -.35f : .35f,
				-0.1f, 1);
		Vec3d barrelPos = player.getEyePosition(partialTicks)
				.add(barrelPosNoTransform.rotatePitch(pitch).rotateYaw(yaw));
		return barrelPos;
	}

	@SubscribeEvent(priority = EventPriority.HIGHEST)
	public static void onBlockBroken(BreakEvent event) {
		PlayerEntity player = event.getPlayer();
		if (player == null)
			return;
		if (!AllItems.PLACEMENT_HANDGUN.typeOf(player.getHeldItemMainhand()))
			return;

		if (event.getState().isNormalCube(player.world, event.getPos())) {
			player.getHeldItemMainhand().getTag().put("BlockUsed", NBTUtil.writeBlockState(event.getState()));
		}
		event.setCanceled(true);
	}

	@SubscribeEvent
	public static void onClientTick(ClientTickEvent event) {
		if (event.phase == Phase.START)
			return;
		if (cachedBeams == null)
			cachedBeams = new LinkedList<>();
		ClientWorld world = Minecraft.getInstance().world;
		if (world == null)
			return;
		ClientPlayerEntity player = Minecraft.getInstance().player;
		if (player == null)
			return;
		cachedBeams.removeIf(b -> b.itensity < .1f);
		cachedBeams.forEach(b -> b.itensity *= .4f);

		lastLeftHandAnimation = leftHandAnimation;
		lastRightHandAnimation = rightHandAnimation;
		leftHandAnimation *= 0.8f;
		rightHandAnimation *= 0.8f;
	}

	@SubscribeEvent
	public static void onRenderWorld(RenderWorldLastEvent event) {
		if (cachedBeams == null || cachedBeams.isEmpty())
			return;

		cachedBeams.forEach(beam -> {
			TessellatorHelper.prepareForDrawing();
			GlStateManager.disableTexture();
			GlStateManager.lineWidth(beam.itensity * 40);

			BufferBuilder bufferBuilder = Tessellator.getInstance().getBuffer();
			bufferBuilder.begin(GL11.GL_LINE_STRIP, DefaultVertexFormats.POSITION);
			bufferBuilder.pos(beam.getStart().x, beam.getStart().y, beam.getStart().z).endVertex();
			bufferBuilder.pos(beam.end.x, beam.end.y, beam.end.z).endVertex();
			Tessellator.getInstance().draw();

			GlStateManager.lineWidth(1);
			GlStateManager.enableTexture();
			TessellatorHelper.cleanUpAfterDrawing();
		});
	}

	public static void shoot(Hand hand) {
		ClientPlayerEntity player = Minecraft.getInstance().player;
		boolean rightHand = hand == Hand.MAIN_HAND ^ player.getPrimaryHand() == HandSide.LEFT;
		if (rightHand) {
			rightHandAnimation = .2f;
			dontReequipRight = false;
		} else {
			leftHandAnimation = .2f;
			dontReequipLeft = false;
		}
		playSound(hand, player.getPosition());
	}

	public static void playSound(Hand hand, BlockPos position) {
		float pitch = hand == Hand.MAIN_HAND ? 2f : 0.9f;
		SoundEvent sound = SoundEvents.BLOCK_NOTE_BLOCK_BASEDRUM;
		Minecraft.getInstance().world.playSound(position, sound, SoundCategory.BLOCKS, 0.8f, pitch, false);
	}

	public static void addBeam(LaserBeam beam) {
		Random r = new Random();
		double x = beam.end.x;
		double y = beam.end.y;
		double z = beam.end.z;
		ClientWorld world = Minecraft.getInstance().world;
		Supplier<Double> randomSpeed = () -> (r.nextDouble() - .5d) * .2f;
		Supplier<Double> randomOffset = () -> (r.nextDouble() - .5d) * .2f;
		for (int i = 0; i < 10; i++) {
			world.addParticle(ParticleTypes.END_ROD, x, y, z, randomSpeed.get(), randomSpeed.get(), randomSpeed.get());
			world.addParticle(ParticleTypes.FIREWORK, x + randomOffset.get(), y + randomOffset.get(),
					z + randomOffset.get(), 0, 0, 0);
		}

		cachedBeams.add(beam);
	}

	@SubscribeEvent
	public static void onRenderPlayerHand(RenderSpecificHandEvent event) {
		if (AllItems.PLACEMENT_HANDGUN.typeOf(event.getItemStack())) {
			Minecraft mc = Minecraft.getInstance();
			boolean rightHand = event.getHand() == Hand.MAIN_HAND ^ mc.player.getPrimaryHand() == HandSide.LEFT;

			GlStateManager.pushMatrix();

			float recoil = rightHand
					? MathHelper.lerp(event.getPartialTicks(), lastRightHandAnimation, rightHandAnimation)
					: MathHelper.lerp(event.getPartialTicks(), lastLeftHandAnimation, leftHandAnimation);

			float equipProgress = event.getEquipProgress();

			if (rightHand && (rightHandAnimation > .01f || dontReequipRight))
				equipProgress = 0;
			if (!rightHand && (leftHandAnimation > .01f || dontReequipLeft))
				equipProgress = 0;

			// Render arm
			float f = rightHand ? 1.0F : -1.0F;
			float f1 = MathHelper.sqrt(event.getSwingProgress());
			float f2 = -0.3F * MathHelper.sin(f1 * (float) Math.PI);
			float f3 = 0.4F * MathHelper.sin(f1 * ((float) Math.PI * 2F));
			float f4 = -0.4F * MathHelper.sin(event.getSwingProgress() * (float) Math.PI);
			GlStateManager.translatef(f * (f2 + 0.64000005F - .1f), f3 + -0.4F + equipProgress * -0.6F,
					f4 + -0.71999997F + .3f + recoil);
			GlStateManager.rotatef(f * 75.0F, 0.0F, 1.0F, 0.0F);
			float f5 = MathHelper.sin(event.getSwingProgress() * event.getSwingProgress() * (float) Math.PI);
			float f6 = MathHelper.sin(f1 * (float) Math.PI);
			GlStateManager.rotatef(f * f6 * 70.0F, 0.0F, 1.0F, 0.0F);
			GlStateManager.rotatef(f * f5 * -20.0F, 0.0F, 0.0F, 1.0F);
			AbstractClientPlayerEntity abstractclientplayerentity = mc.player;
			mc.getTextureManager().bindTexture(abstractclientplayerentity.getLocationSkin());
			GlStateManager.translatef(f * -1.0F, 3.6F, 3.5F);
			GlStateManager.rotatef(f * 120.0F, 0.0F, 0.0F, 1.0F);
			GlStateManager.rotatef(200.0F, 1.0F, 0.0F, 0.0F);
			GlStateManager.rotatef(f * -135.0F, 0.0F, 1.0F, 0.0F);
			GlStateManager.translatef(f * 5.6F, 0.0F, 0.0F);
			GlStateManager.rotatef(f * 40.0F, 0.0F, 1.0F, 0.0F);
			PlayerRenderer playerrenderer = mc.getRenderManager().getRenderer(abstractclientplayerentity);
			GlStateManager.disableCull();
			if (rightHand) {
				playerrenderer.renderRightArm(abstractclientplayerentity);
			} else {
				playerrenderer.renderLeftArm(abstractclientplayerentity);
			}
			GlStateManager.enableCull();
			GlStateManager.popMatrix();

			// Render gun
			GlStateManager.pushMatrix();
			GlStateManager.translatef(f * (f2 + 0.64000005F - .1f), f3 + -0.4F + equipProgress * -0.6F,
					f4 + -0.71999997F - 0.1f + recoil);
			GlStateManager.rotatef(f * f6 * 70.0F, 0.0F, 1.0F, 0.0F);
			GlStateManager.rotatef(f * f5 * -20.0F, 0.0F, 0.0F, 1.0F);

			GlStateManager.translatef(f * -0.1f, 0.1f, -0.4f);
			GlStateManager.rotatef(f * 5.0F, 0.0F, 1.0F, 0.0F);

			FirstPersonRenderer firstPersonRenderer = mc.getFirstPersonRenderer();
			firstPersonRenderer.renderItemSide(mc.player, event.getItemStack(),
					rightHand ? ItemCameraTransforms.TransformType.FIRST_PERSON_RIGHT_HAND
							: ItemCameraTransforms.TransformType.FIRST_PERSON_LEFT_HAND,
					!rightHand);
			GlStateManager.popMatrix();

			event.setCanceled(true);
		}
	}

	public static void dontAnimateItem(Hand hand) {
		boolean rightHand = hand == Hand.MAIN_HAND ^ Minecraft.getInstance().player.getPrimaryHand() == HandSide.LEFT;
		dontReequipRight |= rightHand;
		dontReequipLeft |= !rightHand;
	}

}

==> v0.1.1a.tar.gz: create/src/main/java/com/simibubi/create/modules/curiosities/placementHandgun/BuilderGunHandler.java <==
package com.simibubi.create.modules.curiosities.placementHandgun;

import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.function.Supplier;

import org.lwjgl.opengl.GL11;

import com.mojang.blaze3d.platform.GlStateManager;
import com.simibubi.create.AllItems;
import com.simibubi.create.foundation.utility.TessellatorHelper;

import net.minecraft.client.Minecraft;
import net.minecraft.client.entity.player.AbstractClientPlayerEntity;
import net.minecraft.client.entity.player.ClientPlayerEntity;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.FirstPersonRenderer;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.entity.PlayerRenderer;
import net.minecraft.client.renderer.model.ItemCameraTransforms;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.client.world.ClientWorld;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.particles.ParticleTypes;
import net.minecraft.util.Hand;
import net.minecraft.util.HandSide;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.SoundEvents;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.util.math.Vec3d;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.client.event.RenderSpecificHandEvent;
import net.minecraftforge.client.event.RenderWorldLastEvent;
import net.minecraftforge.event.TickEvent.ClientTickEvent;
import net.minecraftforge.event.TickEvent.Phase;
import net.minecraftforge.event.world.BlockEvent.BreakEvent;
import net.minecraftforge.eventbus.api.EventPriority;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.common.Mod.EventBusSubscriber;

@SuppressWarnings("deprecation")
@EventBusSubscriber(value = Dist.CLIENT)
public class BuilderGunHandler {

	public static List<LaserBeam> cachedBeams;
	public static float leftHandAnimation;
	public static float rightHandAnimation;
	public static float lastLeftHandAnimation;
	public static float lastRightHandAnimation;

	private static boolean dontReequipLeft;
	private static boolean dontReequipRight;

	public static class LaserBeam {
		float itensity;
		Vec3d start;
		Vec3d end;
		boolean follow;
		boolean mainHand;

		public LaserBeam(Vec3d start, Vec3d end) {
			this.start = start;
			this.end = end;
			itensity = 1;
		}

		public LaserBeam followPlayer(boolean follow, boolean mainHand) {
			this.follow = follow;
			this.mainHand = mainHand;
			return this;
		}

		public Vec3d getStart() {
			if (follow)
				return getExactBarrelPos(mainHand);
			return start;
		}
	}

	public static Vec3d getExactBarrelPos(boolean mainHand) {
		float partialTicks = Minecraft.getInstance().getRenderPartialTicks();
		ClientPlayerEntity player = Minecraft.getInstance().player;
		float yaw = (float) ((player.getYaw(partialTicks)) / -180 * Math.PI);
		float pitch = (float) ((player.getPitch(partialTicks)) / -180 * Math.PI);
		boolean rightHand = mainHand == (player.getPrimaryHand() == HandSide.RIGHT);
		float zOffset = ((float) Minecraft.getInstance().gameSettings.fov - 70) / -100;
		Vec3d barrelPosNoTransform = new Vec3d(rightHand ? -.35f : .35f, -0.115f, .75f + zOffset);
		Vec3d barrelPos = player.getEyePosition(partialTicks)
				.add(barrelPosNoTransform.rotatePitch(pitch).rotateYaw(yaw));
		return barrelPos;
	}

	@SubscribeEvent(priority = EventPriority.HIGHEST)
	public static void onBlockBroken(BreakEvent event) {
		PlayerEntity player = event.getPlayer();
		if (player == null)
			return;
		if (!AllItems.PLACEMENT_HANDGUN.typeOf(player.getHeldItemMainhand()))
			return;

		if (event.getState().isNormalCube(player.world, event.getPos())) {
			player.getHeldItemMainhand().getTag().put("BlockUsed", NBTUtil.writeBlockState(event.getState()));
		}
		event.setCanceled(true);
	}

	@SubscribeEvent
	public static void onClientTick(ClientTickEvent event) {
		if (event.phase == Phase.START)
			return;
		if (cachedBeams == null)
			cachedBeams = new LinkedList<>();
		ClientWorld world = Minecraft.getInstance().world;
		if (world == null)
			return;
		ClientPlayerEntity player = Minecraft.getInstance().player;
		if (player == null)
			return;
		cachedBeams.removeIf(b -> b.itensity < .1f);
		cachedBeams.forEach(b -> b.itensity *= .4f);

		lastLeftHandAnimation = leftHandAnimation;
		lastRightHandAnimation = rightHandAnimation;
		leftHandAnimation *= 0.8f;
		rightHandAnimation *= 0.8f;
	}

	@SubscribeEvent
	public static void onRenderWorld(RenderWorldLastEvent event) {
		if (cachedBeams == null || cachedBeams.isEmpty())
			return;

		cachedBeams.forEach(beam -> {
			TessellatorHelper.prepareForDrawing();
			GlStateManager.disableTexture();
			GlStateManager.lineWidth(beam.itensity * 40);

			BufferBuilder bufferBuilder = Tessellator.getInstance().getBuffer();
			bufferBuilder.begin(GL11.GL_LINE_STRIP, DefaultVertexFormats.POSITION);
			bufferBuilder.pos(beam.getStart().x, beam.getStart().y, beam.getStart().z).endVertex();
			bufferBuilder.pos(beam.end.x, beam.end.y, beam.end.z).endVertex();
			Tessellator.getInstance().draw();

			GlStateManager.lineWidth(1);
			GlStateManager.enableTexture();
			TessellatorHelper.cleanUpAfterDrawing();
		});
	}

	public static void shoot(Hand hand) {
		ClientPlayerEntity player = Minecraft.getInstance().player;
		boolean rightHand = hand == Hand.MAIN_HAND ^ player.getPrimaryHand() == HandSide.LEFT;
		if (rightHand) {
			rightHandAnimation = .2f;
			dontReequipRight = false;
		} else {
			leftHandAnimation = .2f;
			dontReequipLeft = false;
		}
		playSound(hand, player.getPosition());
	}

	public static void playSound(Hand hand, BlockPos position) {
		float pitch = hand == Hand.MAIN_HAND ? 2f : 0.9f;
		SoundEvent sound = SoundEvents.BLOCK_NOTE_BLOCK_BASEDRUM;
		Minecraft.getInstance().world.playSound(position, sound, SoundCategory.BLOCKS, 0.8f, pitch, false);
	}

	public static void addBeam(LaserBeam beam) {
		Random r = new Random();
		double x = beam.end.x;
		double y = beam.end.y;
		double z = beam.end.z;
		ClientWorld world = Minecraft.getInstance().world;
		Supplier<Double> randomSpeed = () -> (r.nextDouble() - .5d) * .2f;
		Supplier<Double> randomOffset = () -> (r.nextDouble() - .5d) * .2f;
		for (int i = 0; i < 10; i++) {
			world.addParticle(ParticleTypes.END_ROD, x, y, z, randomSpeed.get(), randomSpeed.get(), randomSpeed.get());
			world.addParticle(ParticleTypes.FIREWORK, x + randomOffset.get(), y + randomOffset.get(),
					z + randomOffset.get(), 0, 0, 0);
		}

		cachedBeams.add(beam);
	}

	@SubscribeEvent
	public static void onRenderPlayerHand(RenderSpecificHandEvent event) {
		if (AllItems.PLACEMENT_HANDGUN.typeOf(event.getItemStack())) {
			Minecraft mc = Minecraft.getInstance();
			boolean rightHand = event.getHand() == Hand.MAIN_HAND ^ mc.player.getPrimaryHand() == HandSide.LEFT;

			GlStateManager.pushMatrix();

			float recoil = rightHand
					? MathHelper.lerp(event.getPartialTicks(), lastRightHandAnimation, rightHandAnimation)
					: MathHelper.lerp(event.getPartialTicks(), lastLeftHandAnimation, leftHandAnimation);

			float equipProgress = event.getEquipProgress();

			if (rightHand && (rightHandAnimation > .01f || dontReequipRight))
				equipProgress = 0;
			if (!rightHand && (leftHandAnimation > .01f || dontReequipLeft))
				equipProgress = 0;

			// Render arm
			float f = rightHand ? 1.0F : -1.0F;
			float f1 = MathHelper.sqrt(event.getSwingProgress());
			float f2 = -0.3F * MathHelper.sin(f1 * (float) Math.PI);
			float f3 = 0.4F * MathHelper.sin(f1 * ((float) Math.PI * 2F));
			float f4 = -0.4F * MathHelper.sin(event.getSwingProgress() * (float) Math.PI);
			GlStateManager.translatef(f * (f2 + 0.64000005F - .1f), f3 + -0.4F + equipProgress * -0.6F,
					f4 + -0.71999997F + .3f + recoil);
			GlStateManager.rotatef(f * 75.0F, 0.0F, 1.0F, 0.0F);
			float f5 = MathHelper.sin(event.getSwingProgress() * event.getSwingProgress() * (float) Math.PI);
			float f6 = MathHelper.sin(f1 * (float) Math.PI);
			GlStateManager.rotatef(f * f6 * 70.0F, 0.0F, 1.0F, 0.0F);
			GlStateManager.rotatef(f * f5 * -20.0F, 0.0F, 0.0F, 1.0F);
			AbstractClientPlayerEntity abstractclientplayerentity = mc.player;
			mc.getTextureManager().bindTexture(abstractclientplayerentity.getLocationSkin());
			GlStateManager.translatef(f * -1.0F, 3.6F, 3.5F);
			GlStateManager.rotatef(f * 120.0F, 0.0F, 0.0F, 1.0F);
			GlStateManager.rotatef(200.0F, 1.0F, 0.0F, 0.0F);
			GlStateManager.rotatef(f * -135.0F, 0.0F, 1.0F, 0.0F);
			GlStateManager.translatef(f * 5.6F, 0.0F, 0.0F);
			GlStateManager.rotatef(f * 40.0F, 0.0F, 1.0F, 0.0F);
			PlayerRenderer playerrenderer = mc.getRenderManager().getRenderer(abstractclientplayerentity);
			GlStateManager.disableCull();
			if (rightHand) {
				playerrenderer.renderRightArm(abstractclientplayerentity);
			} else {
				playerrenderer.renderLeftArm(abstractclientplayerentity);
			}
			GlStateManager.enableCull();
			GlStateManager.popMatrix();

			// Render gun
			GlStateManager.pushMatrix();
			GlStateManager.translatef(f * (f2 + 0.64000005F - .1f), f3 + -0.4F + equipProgress * -0.6F,
					f4 + -0.71999997F - 0.1f + recoil);
			GlStateManager.rotatef(f * f6 * 70.0F, 0.0F, 1.0F, 0.0F);
			GlStateManager.rotatef(f * f5 * -20.0F, 0.0F, 0.0F, 1.0F);

			GlStateManager.translatef(f * -0.1f, 0.1f, -0.4f);
			GlStateManager.rotatef(f * 5.0F, 0.0F, 1.0F, 0.0F);

			FirstPersonRenderer firstPersonRenderer = mc.getFirstPersonRenderer();
			firstPersonRenderer.renderItemSide(mc.player, event.getItemStack(),
					rightHand ? ItemCameraTransforms.TransformType.FIRST_PERSON_RIGHT_HAND
							: ItemCameraTransforms.TransformType.FIRST_PERSON_LEFT_HAND,
					!rightHand);
			GlStateManager.popMatrix();

			event.setCanceled(true);
		}
	}

	public static void dontAnimateItem(Hand hand) {
		boolean rightHand = hand == Hand.MAIN_HAND ^ Minecraft.getInstance().player.getPrimaryHand() == HandSide.LEFT;
		dontReequipRight |= rightHand;
		dontReequipLeft |= !rightHand;
	}

}
